﻿using Buddy.Coroutines;
using Styx;
using Styx.Common;
using Styx.CommonBot;
using Styx.CommonBot.Coroutines;
using Styx.CommonBot.Frames;
using Styx.Plugins;
using Styx.WoWInternals;
using Styx.WoWInternals.DB;
using Styx.WoWInternals.WoWObjects;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;
using System.Windows.Media;

namespace LootHelper
{
    public class LootHelper : HBPlugin
    {
        private static bool _looted = false;
        private static bool _initialized = false;
        private static bool _quit = false;

        public override string Author
        {
            get
            {
                return "Jimmy06";
            }
        }

        public override string Name
        {
            get
            {
                return "Anciant Mana Helper";
            }
        }

        public override Version Version
        {
            get
            {
                return new Version(0, 0, 0, 4);
            }
        }

        public override bool WantButton
        {
            get
            {
                return false;
            }
        }

        public override string ButtonText
        {
            get
            {
                return "";
            }
        }

        public override void OnButtonPress()
        {

        }

        public override void OnEnable()
        {
            if (!_initialized)
            {
                BotEvents.OnBotStopped += OnBotStopped;
                BotEvents.OnBotStarted += OnBotStarted;
                _initialized = true;
            }
        }

        private void OnBotStarted(EventArgs args)
        {
            _quit = false;
        }

        private void OnBotStopped(EventArgs args)
        {
            StopCoroutine();
        }

        public override void OnDisable()
        {
            if (_initialized)
            {
                BotEvents.OnBotStopped -= OnBotStopped;
                BotEvents.OnBotStarted -= OnBotStarted;
                _initialized = false;
            }
        }

        public override void Pulse()
        {
            if (_coroutine == null || _coroutine.IsFinished)
            {
                if (_coroutine != null && !_quit)
                    Log("LootHelper plugin coroutine ended, which it never should! Restarting", LogLevel.Diagnostic);

                _coroutine = new Coroutine(() => MainCoroutine());
            }

            try
            {
                _coroutine.Resume();
            }

            catch (Exception ex)
            {
                Logging.WriteDiagnostic(Colors.Red, ex.Message);
            }

        }

        private Coroutine _coroutine;

        private void StopCoroutine()
        {
            _quit = true;
            if (_coroutine == null)
                return;

            _coroutine.Dispose();
            _coroutine = null;
        }

        private async Task MainCoroutine()
        {
            while (!_quit)
            {
                try
                {
                    Log("Running Logic @ " + DateTime.Now, LogLevel.Diagnostic);
                    if (LootFrame.Instance != null && LootFrame.Instance.IsVisible)
                    {
                        var count = 0;
                        List<UInt32> list = new List<UInt32>();
                        var blacklist = true;
                        while (count < LootFrame.Instance.LootItems)
                        {
                            var itemid = LootFrame.Instance.GetItemId(count);
                            list.Add(itemid);
                            count++;
                        }
                        
                        if (list.Except(new List<UInt32> { 1155 }).Count() == 0 && list.Count() > 0 && blacklist && Blacklist.Contains(LootFrame.Instance.LootingObjectGuid, BlacklistFlags.All) == false)
                            Blacklist.Add(LootFrame.Instance.LootingObjectGuid, BlacklistFlags.All, new TimeSpan(0, 2, 0),
                                    string.Format("Blacklisting {0} for 120 seconds as it only contains Mana",
                                    LootFrame.Instance.LootingObjectGuid));
                    }

                    await Coroutine.Yield();
                }
                catch (Exception ex)
                {
                    Logging.WriteDiagnostic(ex.ToString());
                }
            }
        }

        private async void LOOT_CLOSED(object sender, LuaEventArgs args)
        {
            _looted = true;
            await Task.Delay(SpellManager.GlobalCooldownLeft);
        }

        private void Log(string v, LogLevel l)
        {
            Logging.Write(l, Colors.Aquamarine, string.Format("[LootHelper] {0}", v));
        }
    }
}

